@@ -1,4 +1,5 @@
 // -*- mode: cpp; mode: fold -*-
+// SPDX-License-Identifier: GPL-2.0+
 // Description								/*{{{*/
 /* ######################################################################
 
@@ -8,6 +9,9 @@
    things to strings. They are useful in file parsers, URI handlers and
    especially in APT methods.   
    
+   This file had this historic note, but now includes further changes
+   under the GPL-2.0+:
+
    This source is placed in the Public Domain, do with it what you will
    It was originally written by Jason Gunthorpe <jgg@gpu.srv.ualberta.ca>
    
@@ -22,6 +26,7 @@
 
 #include <algorithm>
 #include <array>
+#include <cstdint>
 #include <iomanip>
 #include <limits>
 #include <locale>
@@ -31,18 +36,18 @@
 #include <string>
 #include <vector>
 
-#include <ctype.h>
-#include <errno.h>
+#include <cctype>
+#include <cerrno>
+#include <cstdarg>
+#include <cstddef>
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
+#include <ctime>
+#include <cwchar>
 #include <iconv.h>
 #include <regex.h>
-#include <stdarg.h>
-#include <stddef.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <time.h>
 #include <unistd.h>
-#include <wchar.h>
 
 #include <apti18n.h>
 									/*}}}*/
@@ -52,43 +57,30 @@ using namespace std;
 // ---------------------------------------------------------------------
 namespace APT {
    namespace String {
-std::string Strip(const std::string &str)
+std::string_view Strip(std::string_view str)
 {
-   // ensure we have at least one character
-   if (str.empty() == true)
-      return str;
-
-   char const * const s = str.c_str();
-   size_t start = 0;
-   for (; isspace(s[start]) != 0; ++start)
-      ; // find the first not-space
-
-   // string contains only whitespaces
-   if (s[start] == '\0')
-      return "";
-
-   size_t end = str.length() - 1;
-   for (; isspace(s[end]) != 0; --end)
-      ; // find the last not-space
-
-   return str.substr(start, end - start + 1);
+   while (!str.empty() && isspace(str[0]))
+      str.remove_prefix(1);
+   while (!str.empty() && isspace(str.back()))
+      str.remove_suffix(1);
+   return str;
 }
 
-bool Endswith(const std::string &s, const std::string &end)
+bool Endswith(const std::string_view &s, const std::string_view &end)
 {
    if (end.size() > s.size())
       return false;
    return (s.compare(s.size() - end.size(), end.size(), end) == 0);
 }
 
-bool Startswith(const std::string &s, const std::string &start)
+bool Startswith(const std::string_view &s, const std::string_view &start)
 {
    if (start.size() > s.size())
       return false;
    return (s.compare(0, start.size(), start) == 0);
 }
 
-std::string Join(std::vector<std::string> list, const std::string &sep)
+std::string Join(std::vector<std::string> list, const std::string_view &sep)
 {
    std::ostringstream oss;
    for (auto it = list.begin(); it != list.end(); it++)
@@ -100,7 +92,7 @@ std::string Join(std::vector<std::string> list, const std::string &sep)
 }
 
 // Returns string display length honoring multi-byte characters
-size_t DisplayLength(StringView str)
+size_t DisplayLength(string_view str)
 {
    size_t len = 0;
 
@@ -244,41 +236,6 @@ char *_strrstrip(char *String)
    return String;
 }
 									/*}}}*/
-// strtabexpand - Converts tabs into 8 spaces				/*{{{*/
-// ---------------------------------------------------------------------
-/* */
-char *_strtabexpand(char *String,size_t Len)
-{
-   for (char *I = String; I != I + Len && *I != 0; I++)
-   {
-      if (*I != '\t')
-	 continue;
-      if (I + 8 > String + Len)
-      {
-	 *I = 0;
-	 return String;
-      }
-
-      /* Assume the start of the string is 0 and find the next 8 char
-         division */
-      int Len;
-      if (String == I)
-	 Len = 1;
-      else
-	 Len = 8 - ((String - I) % 8);
-      Len -= 2;
-      if (Len <= 0)
-      {
-	 *I = ' ';
-	 continue;
-      }
-      
-      memmove(I + Len,I + 1,strlen(I) + 1);
-      for (char *J = I; J + Len != I; *I = ' ', I++);
-   }
-   return String;
-}
-									/*}}}*/
 // ParseQuoteWord - Parse a single word out of a string			/*{{{*/
 // ---------------------------------------------------------------------
 /* This grabs a single word, converts any % escaped characters to their
@@ -499,10 +456,10 @@ string TimeToStr(unsigned long Sec)
 // SubstVar - Substitute a string for another string			/*{{{*/
 // ---------------------------------------------------------------------
 /* This replaces all occurrences of Subst with Contents in Str. */
-string SubstVar(const string &Str,const string &Subst,const string &Contents)
+string SubstVar(const string_view &Str,const string_view &Subst,const string_view &Contents)
 {
    if (Subst.empty() == true)
-      return Str;
+      return std::string{Str};
 
    string::size_type Pos = 0;
    string::size_type OldPos = 0;
@@ -519,7 +476,7 @@ string SubstVar(const string &Str,const string &Subst,const string &Contents)
    }
 
    if (OldPos == 0)
-      return Str;
+      return std::string{Str};
 
    if (OldPos >= Str.length())
       return Temp;
@@ -620,6 +577,73 @@ string Base64Encode(const string &S)
    
    return Final;
 }
+									/*}}}*/
+// Base64Decode - Base64 decoding routine for short strings		/*{{{*/
+// ---------------------------------------------------------------------
+// Taken from mjg59's dpkg code: https://lists.debian.org/debian-dpkg/2018/05/msg00002.html
+std::string Base64Decode(const std::string_view in)
+{
+   constexpr std::array<unsigned char, 256> mime_base64_rank{
+      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
+      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
+      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 62, 255, 255, 255, 63,
+      52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 255, 255, 255, 0, 255, 255,
+      255, 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14,
+      15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 255, 255, 255, 255, 255,
+      255, 26, 27, 28, 29, 30, 31, 32, 33, 34, 35, 36, 37, 38, 39, 40,
+      41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 51, 255, 255, 255, 255, 255,
+      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
+      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
+      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
+      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
+      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
+      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
+      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
+      255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255};
+   std::string out;
+   unsigned char rank;
+   unsigned char last[2];
+   unsigned int v;
+   int i;
+
+   /* convert 4 base64 bytes to 3 normal bytes */
+   v = 0;
+   i = 0;
+
+   last[0] = last[1] = 0;
+
+   /* we use the sign in the state to determine if we got a padding character
+   in the previous sequence */
+   if (i < 0)
+   {
+      i = -i;
+      last[0] = '=';
+   }
+
+   for (const unsigned char c : in)
+   {
+      rank = mime_base64_rank[c];
+      if (rank != 0xff)
+      {
+	 last[1] = last[0];
+	 last[0] = c;
+	 v = (v << 6) | rank;
+	 i++;
+	 if (i == 4)
+	 {
+	    out += v >> 16;
+	    if (last[1] != '=')
+	       out += v >> 8;
+	    if (last[0] != '=')
+	       out += v;
+	    i = 0;
+	 }
+      }
+   }
+
+   return out;
+}
+
 									/*}}}*/
 // stringcmp - Arbitrary string compare					/*{{{*/
 // ---------------------------------------------------------------------
@@ -859,7 +883,7 @@ string TimeRFC1123(time_t Date, bool const NumericTimezone)
    auto const posix = std::locale::classic();
    std::ostringstream datestr;
    datestr.imbue(posix);
-   APT::StringView const fmt("%a, %d %b %Y %H:%M:%S");
+   std::string_view const fmt("%a, %d %b %Y %H:%M:%S");
    std::use_facet<std::time_put<char>>(posix).put(
                     std::ostreambuf_iterator<char>(datestr),
                     datestr, ' ', &Conv, fmt.data(), fmt.data() + fmt.size());
@@ -1125,20 +1149,6 @@ bool RFC1123StrToTime(std::string const &str,time_t &time)
    return true;
 }
 									/*}}}*/
-// FTPMDTMStrToTime - Converts a ftp modification date into a time_t	/*{{{*/
-// ---------------------------------------------------------------------
-/* */
-bool FTPMDTMStrToTime(const char* const str,time_t &time)
-{
-   struct tm Tm;
-   // MDTM includes no whitespaces but recommend and ignored by strptime
-   if (strptime(str, "%Y %m %d %H %M %S", &Tm) == NULL)
-      return false;
-
-   time = timegm(&Tm);
-   return true;
-}
-									/*}}}*/
 // StrToNum - Convert a fixed length string to a number			/*{{{*/
 // ---------------------------------------------------------------------
 /* This is used in decoding the crazy fixed length string headers in
@@ -1236,7 +1246,7 @@ static int HexDigit(int c)
 // Hex2Num - Convert a long hex number into a buffer			/*{{{*/
 // ---------------------------------------------------------------------
 /* The length of the buffer must be exactly 1/2 the length of the string. */
-bool Hex2Num(const APT::StringView Str,unsigned char *Num,unsigned int Length)
+bool Hex2Num(const std::string_view Str,unsigned char *Num,unsigned int Length)
 {
    if (Str.length() != Length*2)
       return false;
@@ -1305,13 +1315,13 @@ bool TokSplitString(char Tok,char *Input,char **List,
 /* This can be used to split a given string up into a vector, so the
    propose is the same as in the method above and this one is a bit slower
    also, but the advantage is that we have an iteratable vector */
-vector<string> VectorizeString(string const &haystack, char const &split)
+vector<string> VectorizeString(string_view const &haystack, char const &split)
 {
    vector<string> exploded;
    if (haystack.empty() == true)
       return exploded;
-   string::const_iterator start = haystack.begin();
-   string::const_iterator end = start;
+   auto start = haystack.begin();
+   auto end = start;
    do {
       for (; end != haystack.end() && *end != split; ++end);
       exploded.push_back(string(start, end));
@@ -1324,7 +1334,7 @@ vector<string> VectorizeString(string const &haystack, char const &split)
 // ---------------------------------------------------------------------
 /* See header for details.
  */
-vector<string> StringSplit(std::string const &s, std::string const &sep,
+vector<string> StringSplit(std::string_view const &s, std::string_view const &sep,
                            unsigned int maxsplit)
 {
    vector<string> split;
@@ -1338,8 +1348,8 @@ vector<string> StringSplit(std::string const &s, std::string const &sep,
    while (pos != string::npos)
    {
       pos = s.find(sep, start);
-      split.push_back(s.substr(start, pos-start));
-      
+      split.emplace_back(s.substr(start, pos-start));
+
       // if maxsplit is reached, the remaining string is the last item
       if(split.size() >= maxsplit)
       {
@@ -1417,7 +1427,7 @@ unsigned long RegexChoice(RxChoiceList *Rxs,const char **ListBegin,
    and to allow reordering of parameters */
 bool iovprintf(std::ostream &out, const char *format,
 		      va_list &args, ssize_t &size) {
-   auto S = std::unique_ptr<char,decltype(&free)>{static_cast<char*>(malloc(size)), &free};
+   auto S = std::unique_ptr<char,FreeDeleter>{static_cast<char*>(malloc(size))};
    ssize_t const n = vsnprintf(S.get(), size, format, args);
    if (n > -1 && n < size) {
       out << S.get();
