@@ -600,7 +600,7 @@ void pkgDPkgPM::ProcessDpkgStatusLine(char *line)
 
    // build the (prefix, pkgname, action) tuple, position of this
    // is different for "processing" or "status" messages
-   std::string prefix = APT::String::Strip(list[0]);
+   auto prefix = APT::String::Strip(list[0]);
    std::string pkgname;
    std::string action;
 
@@ -761,7 +761,7 @@ void pkgDPkgPM::ProcessDpkgStatusLine(char *line)
 	       char* buf = NULL;
 	       size_t bufsize = 0;
 	       if (getline(&buf, &bufsize, dpkg) != -1)
-		  (pkgname += ':') += buf;
+		  pkgname.append(":").append(buf);
 	       free(buf);
 	       fclose(dpkg);
 	    }
@@ -1087,6 +1087,8 @@ bool pkgDPkgPM::OpenLog()
       std::string RequestingUser = AptHistoryRequestingUser();
       if (RequestingUser != "")
          WriteHistoryTag("Requested-By", RequestingUser);
+      if (auto comment = _config->Find("APT::History::Comment"); not comment.empty())
+	 WriteHistoryTag("Comment", comment);
       WriteHistoryTag("Install", install);
       WriteHistoryTag("Reinstall", reinstall);
       WriteHistoryTag("Upgrade", upgrade);
@@ -1486,7 +1488,7 @@ public:
    auto begin() const { return args.cbegin(); }
    auto end() const { return args.cend(); }
    auto& front() const { return args.front(); }
-   APT_NORETURN void execute(char const *const errmsg) {
+   [[noreturn]] void execute(char const *const errmsg) {
       args.push_back(nullptr);
       execvp(args.front(), &args.front());
       std::cerr << errmsg << std::endl;
@@ -1771,7 +1773,6 @@ bool pkgDPkgPM::Go(APT::Progress::PackageManager *progress)
 
    // this loop is runs once per dpkg operation
    vector<Item>::const_iterator I = List.cbegin();
-   BuildDpkgCall Args;
    while (I != List.end())
    {
       // Do all actions with the same Op in one run
@@ -1793,7 +1794,7 @@ bool pkgDPkgPM::Go(APT::Progress::PackageManager *progress)
       else
 	 J = std::find_if(J, List.cend(), [&J](Item const &I) { return I.Op != J->Op; });
 
-      Args.clearCallArguments();
+      BuildDpkgCall Args;
       Args.reserve((J - I) + 10);
 
       int fd[2];
@@ -1875,7 +1876,7 @@ bool pkgDPkgPM::Go(APT::Progress::PackageManager *progress)
 	    {
 	       if (I->File[0] != '/')
 		  return _error->Error("Internal Error, Pathname to install is not absolute '%s'",I->File.c_str());
-	       auto file = flNotDir(I->File);
+	       std::string file{flNotDir(I->File)};
 	       if (flExtension(file) != "deb")
 		  file.append(".deb");
 	       std::string linkpath;
@@ -2243,14 +2244,6 @@ void pkgDPkgPM::Reset()
    List.erase(List.begin(),List.end());
 }
 
-template <class F>
-struct AptScopeWrapper {
-   F func;
-   ~AptScopeWrapper() { func(); }
-};
-template <class F>
-AptScopeWrapper(F) -> AptScopeWrapper<F>;
-
 static void CopyIndented(const char *header, FILE *from, FILE *to)
 {
    if (!from)
@@ -2259,7 +2252,7 @@ static void CopyIndented(const char *header, FILE *from, FILE *to)
    fputs(header, to);
    char *line{};
    size_t linelen;
-   AptScopeWrapper line_deleter{[&] { free(line); }};
+   DEFER([&] { free(line); });
    while (getline(&line, &linelen, from) != -1)
       fprintf(to, " %s", line);
 }
@@ -2376,7 +2369,7 @@ void pkgDPkgPM::WriteApportReport(const char *pkgpath, const char *errormsg)
    pkgver = Ver.VerStr() == NULL ? "unknown" : Ver.VerStr();
 
    // if the file exists already, we check:
-   // - if it was reported already (touched by apport). 
+   // - if it was reported already (touched by apport).
    //   If not, we do nothing, otherwise
    //    we overwrite it. This is the same behaviour as apport
    // - if we have a report with the same pkgversion already
@@ -2393,10 +2386,10 @@ void pkgDPkgPM::WriteApportReport(const char *pkgpath, const char *errormsg)
 
       char *line{};
       size_t linelen;
-      AptScopeWrapper line_deleter{[&] { free(line); }};
+      DEFER([&] { free(line); });
       // check if the existing report is the same version
       report = fopen(reportfile.c_str(),"r");
-      AptScopeWrapper report_deleter{[&] { fclose(report); }};
+      DEFER([&] { fclose(report); });
       while(getline(&line, &linelen, report) != -1)
       {
 	 char pkgname[255], version[255];
@@ -2411,7 +2404,7 @@ void pkgDPkgPM::WriteApportReport(const char *pkgpath, const char *errormsg)
    report = fopen(reportfile.c_str(),"w");
    if(report == NULL)
       return;
-   AptScopeWrapper report_deleter{[&] { fclose(report); }};
+   DEFER([&] { fclose(report); });
    if(_config->FindB("DPkgPM::InitialReportOnly",false) == true)
       chmod(reportfile.c_str(), 0);
    else
