@@ -24,7 +24,7 @@
 #include <apt-pkg/pkgsystem.h>
 #include <apt-pkg/progress.h>
 #include <apt-pkg/sourcelist.h>
-#include <apt-pkg/string_view.h>
+#include <apt-pkg/strutl.h>
 #include <apt-pkg/version.h>
 
 #include <algorithm>
@@ -68,6 +68,8 @@ pkgCacheGenerator::pkgCacheGenerator(DynamicMMap *pMap,OpProgress *Prog) :
 		    Map(*pMap), Cache(pMap,false), Progress(Prog),
 		     CurrentRlsFile(nullptr), CurrentFile(nullptr), d(nullptr)
 {
+   // Reserve some space for version extra data, to avoid lots of small resizes.
+   VersionExtra.reserve(32 * 1024);
 }
 bool pkgCacheGenerator::Start()
 {
@@ -396,8 +398,8 @@ bool pkgCacheGenerator::MergeListVersion(ListParser &List, pkgCache::PkgIterator
 	    if (List.SameVersion(Hash, Ver))
 	    {
                // We do not have SHA256 for both, so we cannot compare them, trust the call from SameVersion()
-               if (ListSHA256.empty() || VersionExtra[Ver->ID].SHA256[0] == 0)
-                  break;
+	       if (ListSHA256.empty() || Ver->ID >= VersionExtra.size() || VersionExtra[Ver->ID].SHA256[0] == 0)
+		  break;
                // We have SHA256 for both, so they must match.
                if (ListSHA256 == std::string_view(VersionExtra[Ver->ID].SHA256, 64))
                   break;
@@ -869,11 +871,6 @@ bool pkgCacheGenerator::NewFileVer(pkgCache::VerIterator &Ver,
    auto const Size = List.Size();
    if (Cache.HeaderP->MaxVerFileSize < Size)
       Cache.HeaderP->MaxVerFileSize = Size;
-#if APT_PKG_ABI <= 600
-   APT_IGNORE_DEPRECATED_PUSH
-   VF->Size = Size;
-   APT_IGNORE_DEPRECATED_POP
-#endif
    Cache.HeaderP->VerFileCount++;
 
    return true;
@@ -899,6 +896,11 @@ map_pointer<pkgCache::Version> pkgCacheGenerator::NewVersion(pkgCache::VerIterat
    Ver->d = d;
    if (not Ver.PhasedUpdatePercentage(100))
       abort();
+   auto SourceVersion = AllocateInMap<pkgCache::SourceVersion>(); // sequence point so Ver can be moved if needed
+   Ver->SourceVersion = SourceVersion;
+   Ver.SourceVersion()->ID = Cache.HeaderP->SourceVersionCount++;
+   if (not Ver.SourceVersion())
+      abort();
 
    //Dynamic<pkgCache::VerIterator> DynV(Ver); // caller MergeListVersion already takes care of it
    Ver->NextVer = Next;
@@ -975,11 +977,6 @@ bool pkgCacheGenerator::NewFileDesc(pkgCache::DescIterator &Desc,
    auto const Size = List.Size();
    if (Cache.HeaderP->MaxDescFileSize < Size)
       Cache.HeaderP->MaxDescFileSize = Size;
-#if APT_PKG_ABI <= 600
-   APT_IGNORE_DEPRECATED_PUSH
-   DF->Size = Size;
-   APT_IGNORE_DEPRECATED_POP
-#endif
    Cache.HeaderP->DescFileCount++;
 
    return true;
