@@ -39,6 +39,7 @@ struct PackageSortAlphabetic						/*{{{*/
 
 class PackageNameMatcher : public Matcher
 {
+   static constexpr const char *const isfnmatch_strict = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789+-.:*";
    pkgCacheFile &cacheFile;
   public:
    explicit PackageNameMatcher(pkgCacheFile &cacheFile, const char **patterns)
@@ -48,12 +49,12 @@ class PackageNameMatcher : public Matcher
       {
          std::string pattern = patterns[i];
          APT::CacheFilter::Matcher *cachefilter = NULL;
-	 if (pattern.size() > 0 && (pattern[0] == '?' || pattern[0] == '~'))
-	    cachefilter = APT::CacheFilter::ParsePattern(pattern, &cacheFile).release();
-         else if(_config->FindB("APT::Cmd::Use-Regexp", false) == true)
+         if(_config->FindB("APT::Cmd::Use-Regexp", false) == true)
             cachefilter = new APT::CacheFilter::PackageNameMatchesRegEx(pattern);
-         else
+         else if (pattern.find_first_not_of(isfnmatch_strict) == std::string::npos)
             cachefilter = new APT::CacheFilter::PackageNameMatchesFnmatch(pattern);
+	 else
+	    cachefilter = APT::CacheFilter::ParsePattern(pattern, &cacheFile).release();
 
          if (cachefilter == nullptr) {
             return;
@@ -62,12 +63,12 @@ class PackageNameMatcher : public Matcher
          filters.push_back(cachefilter);
       }
    }
-   virtual ~PackageNameMatcher()
+   ~PackageNameMatcher() override
    {
       for(J=filters.begin(); J != filters.end(); ++J)
          delete *J;
    }
-   virtual bool operator () (const pkgCache::PkgIterator &P) APT_OVERRIDE
+   bool operator()(const pkgCache::PkgIterator &P) override
    {
       for(J=filters.begin(); J != filters.end(); ++J)
       {
@@ -130,6 +131,10 @@ bool DoList(CommandLine &Cmd)
    GetLocalitySortedVersionSet(CacheFile, &bag, matcher, &progress);
    bool const ShowAllVersions = _config->FindB("APT::Cmd::All-Versions", false);
    std::map<std::string, std::string> output_map;
+
+   if (not InitOutputPager())
+      return false;
+
    for (LocalitySortedVersionSet::iterator V = bag.begin(); V != bag.end(); ++V)
    {
       std::stringstream outs;
