@@ -1,14 +1,14 @@
 // -*- mode: cpp; mode: fold -*-
 // Description								/*{{{*/
 /* ######################################################################
-   
+
    APT Extract Templates - Program to extract debconf config and template
                            files
 
-   This is a simple program to extract config and template information 
+   This is a simple program to extract config and template information
    from Debian packages. It can be used to speed up the preconfiguration
    process for debconf-enabled packages
-   
+
    ##################################################################### */
 									/*}}}*/
 // Include Files							/*{{{*/
@@ -28,16 +28,17 @@
 #include <apt-pkg/pkgsystem.h>
 #include <apt-pkg/sourcelist.h>
 #include <apt-pkg/strutl.h>
+#include <apt-pkg/tagfile-keys.h>
 #include <apt-pkg/tagfile.h>
 #include <apt-pkg/version.h>
 
 #include <apt-private/private-cmndline.h>
 #include <apt-private/private-main.h>
 
+#include <cstdio>
+#include <cstdlib>
+#include <cstring>
 #include <iostream>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
 #include <unistd.h>
 
 #include "apt-extracttemplates.h"
@@ -74,7 +75,7 @@ DebFile::~DebFile()
 string DebFile::GetInstalledVer(const string &package)
 {
 	pkgCache::PkgIterator Pkg = Cache->FindPkg(package);
-	if (Pkg.end() == false) 
+	if (Pkg.end() == false)
 	{
 		pkgCache::VerIterator V = Pkg.CurrentVer();
 		if (V.end() == false)
@@ -119,8 +120,8 @@ bool DebFile::DoItem(Item &I, int &Fd)
 		Config = new char[I.Size+1];
 		Config[I.Size] = 0;
 		Which = IsConfig;
-		Fd = -2; 
-	} 
+		Fd = -2;
+	}
 	else if (strcmp(I.Name, "templates") == 0)
 	{
 		delete [] Template;
@@ -128,8 +129,8 @@ bool DebFile::DoItem(Item &I, int &Fd)
 		Template[I.Size] = 0;
 		Which = IsTemplate;
 		Fd = -2;
-	} 
-	else 
+	}
+	else
 	{
 		// Ignore it
 		Fd = -1;
@@ -170,11 +171,11 @@ bool DebFile::ParseInfo()
 	if (Section.Scan(Control, ControlLen) == false)
 		return false;
 
-	Package = Section.FindS("Package");
+	Package = Section.Find(pkgTagSection::Key::Package);
 	Version = GetInstalledVer(Package);
 
 	const char *Start, *Stop;
-	if (Section.Find("Depends", Start, Stop) == true)
+	if (Section.Find(pkgTagSection::Key::Depends, Start, Stop))
 	{
 		while (1)
 		{
@@ -191,8 +192,8 @@ bool DebFile::ParseInfo()
 			if (Start == Stop) break;
 		}
 	}
-	
-	if (Section.Find("Pre-Depends", Start, Stop) == true)
+
+	if (Section.Find(pkgTagSection::Key::Pre_Depends, Start, Stop))
 	{
 		while (1)
 		{
@@ -209,7 +210,7 @@ bool DebFile::ParseInfo()
 			if (Start == Stop) break;
 		}
 	}
-	
+
 	return true;
 }
 									/*}}}*/
@@ -248,7 +249,7 @@ static void WriteConfig(const DebFile &file)
 
 	if (templatefile.empty() == true || configscript.empty() == true)
 		return;
-	cout << file.Package << " " << file.Version << " " 
+	cout << file.Package << " " << file.Version << " "
 	     << templatefile << " " << configscript << endl;
 }
 									/*}}}*/
@@ -256,7 +257,7 @@ static void WriteConfig(const DebFile &file)
 // ---------------------------------------------------------------------
 /* */
 static bool Go(CommandLine &CmdL)
-{	
+{
 	// Initialize the apt cache
 	MMap *Map = 0;
 	pkgSourceList List;
@@ -302,17 +303,17 @@ static bool Go(CommandLine &CmdL)
 			if (file.PreDepVer != "" &&
 			    DebFile::Cache->VS->CheckDep(debconfver.c_str(),
 			                file.PreDepOp,file.PreDepVer.c_str()
-							 ) == false) 
+							 ) == false)
 				continue;
 
 			WriteConfig(file);
 		}
 	}
-	
+
 
 	delete Map;
 	delete DebFile::Cache;
-	
+
 	return !_error->PendingError();
 }
 									/*}}}*/
