@@ -284,9 +284,9 @@ struct HttpConnectFd final : public MethodFd
    std::unique_ptr<MethodFd> UnderlyingFd;
    std::string Buffer;
 
-   int Fd() APT_OVERRIDE { return UnderlyingFd->Fd(); }
+   int Fd() override { return UnderlyingFd->Fd(); }
 
-   ssize_t Read(void *buf, size_t count) APT_OVERRIDE
+   ssize_t Read(void *buf, size_t count) override
    {
       if (!Buffer.empty())
       {
@@ -299,17 +299,17 @@ struct HttpConnectFd final : public MethodFd
 
       return UnderlyingFd->Read(buf, count);
    }
-   ssize_t Write(void *buf, size_t count) APT_OVERRIDE
+   ssize_t Write(void *buf, size_t count) override
    {
       return UnderlyingFd->Write(buf, count);
    }
 
-   int Close() APT_OVERRIDE
+   int Close() override
    {
       return UnderlyingFd->Close();
    }
 
-   bool HasPending() APT_OVERRIDE
+   bool HasPending() override
    {
       return !Buffer.empty();
    }
@@ -428,9 +428,7 @@ ResultState HttpServerState::Open()
    Out.Reset();
    Persistent = true;
 
-#ifdef HAVE_GNUTLS
    bool tls = (ServerName.Access == "https" || APT::String::Endswith(ServerName.Access, "+https"));
-#endif
 
    // Determine the proxy setting
    // Used to run AutoDetectProxy(ServerName) here, but we now send a Proxy
@@ -455,7 +453,6 @@ ResultState HttpServerState::Open()
 	   {
 	      char *result = getenv("http_proxy");
 	      Proxy = result ? result : "";
-#ifdef HAVE_GNUTLS
 	      if (tls == true)
 	      {
 		 char *result = getenv("https_proxy");
@@ -464,7 +461,6 @@ ResultState HttpServerState::Open()
 		    Proxy = result;
 		 }
 	      }
-#endif
 	   }
    }
    
@@ -478,13 +474,8 @@ ResultState HttpServerState::Open()
    if (Proxy.empty() == false)
       Owner->AddProxyAuth(Proxy, ServerName);
 
-#ifdef HAVE_GNUTLS
    auto const DefaultService = tls ? "https" : "http";
    auto const DefaultPort = tls ? 443 : 80;
-#else
-   auto const DefaultService = "http";
-   auto const DefaultPort = 80;
-#endif
    if (Proxy.Access == "socks5h")
    {
       auto result = Connect(Proxy.Host, Proxy.Port, "socks", 1080, ServerFd, TimeOut, Owner);
@@ -518,15 +509,12 @@ ResultState HttpServerState::Open()
 	    Port = Proxy.Port;
 	 Host = Proxy.Host;
 
-#ifdef HAVE_GNUTLS
 	 if (Proxy.Access == "https" && Port == 0)
 	    Port = 443;
-#endif
       }
       auto result = Connect(Host, Port, DefaultService, DefaultPort, ServerFd, TimeOut, Owner);
       if (result != ResultState::SUCCESSFUL)
 	 return result;
-#ifdef HAVE_GNUTLS
       if (Host == Proxy.Host && Proxy.Access == "https")
       {
 	 aptConfigWrapperForMethods ProxyConf{std::vector<std::string>{"http", "https"}};
@@ -541,13 +529,10 @@ ResultState HttpServerState::Open()
 	 if (result != ResultState::SUCCESSFUL)
 	    return result;
       }
-#endif
    }
 
-#ifdef HAVE_GNUTLS
    if (tls)
       return UnwrapTLS(ServerName.Host, ServerFd, TimeOut, Owner, Owner);
-#endif
 
    return ResultState::SUCCESSFUL;
 }
@@ -1039,7 +1024,7 @@ BaseHttpMethod::DealWithHeadersResult HttpMethod::DealWithHeaders(FetchResult &R
 									/*}}}*/
 HttpMethod::HttpMethod(std::string &&pProg) : BaseHttpMethod(std::move(pProg), "1.2", Pipeline | SendConfig | SendURIEncoded) /*{{{*/
 {
-   SeccompFlags = aptMethod::BASE | aptMethod::NETWORK;
+   SeccompFlags = aptMethod::BASE | aptMethod::NETWORK | aptMethod::DIRECTORY;
 
    auto addName = std::inserter(methodNames, methodNames.begin());
    if (Binary != "http")
@@ -1060,7 +1045,7 @@ int main(int, const char *argv[])
    // ignore SIGPIPE, this can happen on write() if the socket
    // closes the connection (this is dealt with via ServerDie())
    signal(SIGPIPE, SIG_IGN);
-   std::string Binary = flNotDir(argv[0]);
+   std::string Binary{flNotDir(argv[0])};
    if (Binary.find('+') == std::string::npos && Binary != "https" && Binary != "http")
       Binary.append("+http");
    return HttpMethod(std::move(Binary)).Loop();
